#include <16F628.h>
#use delay(clock=4000000)
#fuses HS,INTRC,NOWDT,NOMCLR,NOBROWNOUT,NOLVP
#use rs232(baud=1200,parity=N,xmit=PIN_A3,rcv=PIN_A2)

/*
Notes:

This is a simple program to decode a rotary optical encoder to step/dir output. Useful for manual operation of stepper motor drivers. 
PICC PCW compiler version 3.242
Minimal parts count using Pic 16F628 internal 4Mhz system clock, just need a external +5volt supply. 
Decodes the optical encoder at 1x resolution.
Pin B4 and B3 are optical encoder inputs.
Pin A0 is Direction output.
Pin A1 is Step output, active low with 5us pulse width
You can change stepincrement value to set the number of step output pulses for one optical encoder count.
To high of a stepincrement value or encoder speed will cause missing motor steps due to no accelleration ramp. 
The serial port will output the encoder position at 1200baud using pin A3. 
Program tested and verified to work with our microstep driver board. 

The quadrature decoding code was originally found on the net and modified by me. I do not know who wrote it.
Usual disclaimer: Use at your own risk, no guarantees this program works as advertised. 

email: jimf at embeddedtronics.com
www.embeddedtronics.com
*/

#byte   portb  =  6
signed int32 position=0;
int stepincrement = 5;

#INT_RB
void quad1x(void) {
   //Pin B4 and B3 encoder inputs
   static int old;
   static int new;
   static int value;
   static int count;
   //Here I have an edge on one of the quadrature inputs
   new=portb;
   value=new^old;
   count=0;
   //'value', now has the bit set, which has changed
   if (value & 0x10) {
      //Here the low bit has changed
      if (new & 0x10) {
         //Here a rising edge on A
         if (new & 0x8)
            {--position;
             output_low(PIN_A0);}  //set direction pin low
            else {
            ++position;
            output_high(PIN_A0);}  //set direction pin high

            Do {
            output_low(PIN_A1);   //pulse step pin
            delay_us(5);
            output_high(PIN_A1);
            count++; }
            while (count<stepincrement+1);
      }
   }
    old=new;
}

void main() {

enable_interrupts(INT_RB);      // turn on port b interrupt on change
enable_interrupts(GLOBAL);

port_b_pullups(true);
set_tris_b(0b11111111);

do {

      printf("%ld\n\r",position);
      delay_ms(300);
      } while (TRUE);

}
